@@ -29,10 +29,20 @@
 int Curl_inet_pton(int, const char *, void *);
 
 #ifdef HAVE_INET_PTON
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
+#if defined(__AMIGA__)
+#define Curl_inet_pton(x,y,z) inet_pton(x,(unsigned char *)y,z)
+#else
 #define Curl_inet_pton(x,y,z) inet_pton(x,y,z)
 #endif
+#endif
 
 #endif /* HEADER_CURL_INET_PTON_H */
