@@ -24,9 +24,10 @@
 #include "tool_setup.h"
 
 #include "strcase.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
-#define ENABLE_CURLX_PRINTF
-/* use our own printf() functions */
 #include "curlx.h"
 
 #include "tool_cfgable.h"
@@ -47,7 +48,7 @@ static char *parse_filename(const char *ptr, size_t len);
 #else
 #define BOLD "\x1b[1m"
 /* Switch off bold by setting "all attributes off" since the explicit
-   bold-off code (21) isn't supported everywhere - like in the mac
+   bold-off code (21) is not supported everywhere - like in the mac
    Terminal. */
 #define BOLDOFF "\x1b[0m"
 /* OSC 8 hyperlink escape sequence */
@@ -61,6 +62,25 @@ static void write_linked_location(CURL *curl, const char *location,
     size_t loclen, FILE *stream);
 #endif
 
+int tool_write_headers(struct HdrCbData *hdrcbdata, FILE *stream)
+{
+  struct curl_slist *h = hdrcbdata->headlist;
+  int rc = 1;
+  while(h) {
+    /* not "handled", just show it */
+    size_t len = strlen(h->data);
+    if(len != fwrite(h->data, 1, len, stream))
+      goto fail;
+    h = h->next;
+  }
+  rc = 0; /* success */
+fail:
+  curl_slist_free_all(hdrcbdata->headlist);
+  hdrcbdata->headlist = NULL;
+  return rc;
+}
+
+
 /*
 ** callback for CURLOPT_HEADERFUNCTION
 */
@@ -102,15 +122,29 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
     if(rc != cb)
       return rc;
     /* flush the stream to send off what we got earlier */
-    (void)fflush(heads->stream);
+    if(fflush(heads->stream)) {
+      errorf(per->config->global, "Failed writing headers to %s",
+             per->config->headerfile);
+      return CURL_WRITEFUNC_ERROR;
+    }
   }
 
-  /*
-   * Write etag to file when --etag-save option is given.
-   */
-  if(per->config->etag_save_file && etag_save->stream) {
-    /* match only header that start with etag (case insensitive) */
-    if(curl_strnequal(str, "etag:", 5)) {
+  curl_easy_getinfo(per->curl, CURLINFO_SCHEME, &scheme);
+  scheme = proto_token(scheme);
+  if((scheme == proto_http || scheme == proto_https)) {
+    long response = 0;
+    curl_easy_getinfo(per->curl, CURLINFO_RESPONSE_CODE, &response);
+
+    if((response/100 != 2) && (response/100 != 3))
+      /* only care about etag and content-disposition headers in 2xx and 3xx
+         responses */
+      ;
+    /*
+     * Write etag to file when --etag-save option is given.
+     */
+    else if(per->config->etag_save_file && etag_save->stream &&
+            /* match only header that start with etag (case insensitive) */
+            checkprefix("etag:", str)) {
       const char *etag_h = &str[5];
       const char *eot = end - 1;
       if(*eot == '\n') {
@@ -121,6 +155,19 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
 
         if(eot >= etag_h) {
           size_t etag_length = eot - etag_h + 1;
+          /*
+           * Truncate the etag save stream, it can have an existing etag value.
+           */
+#ifdef HAVE_FTRUNCATE
+          if(ftruncate(fileno(etag_save->stream), 0)) {
+            return CURL_WRITEFUNC_ERROR;
+          }
+#else
+          if(fseek(etag_save->stream, 0, SEEK_SET)) {
+            return CURL_WRITEFUNC_ERROR;
+          }
+#endif
+
           fwrite(etag_h, size, etag_length, etag_save->stream);
           /* terminate with newline */
           fputc('\n', etag_save->stream);
@@ -128,69 +175,99 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
         }
       }
     }
-  }
 
-  /*
-   * This callback sets the filename where output shall be written when
-   * curl options --remote-name (-O) and --remote-header-name (-J) have
-   * been simultaneously given and additionally server returns an HTTP
-   * Content-Disposition header specifying a filename property.
-   */
+    /*
+     * This callback sets the filename where output shall be written when
+     * curl options --remote-name (-O) and --remote-header-name (-J) have
+     * been simultaneously given and additionally server returns an HTTP
+     * Content-Disposition header specifying a filename property.
+     */
 
-  curl_easy_getinfo(per->curl, CURLINFO_SCHEME, &scheme);
-  scheme = proto_token(scheme);
-  if(hdrcbdata->honor_cd_filename &&
-     (cb > 20) && checkprefix("Content-disposition:", str) &&
-     (scheme == proto_http || scheme == proto_https)) {
-    const char *p = str + 20;
+    else if(hdrcbdata->honor_cd_filename) {
+      if((cb > 20) && checkprefix("Content-disposition:", str)) {
+        const char *p = str + 20;
 
-    /* look for the 'filename=' parameter
-       (encoded filenames (*=) are not supported) */
-    for(;;) {
-      char *filename;
-      size_t len;
+        /* look for the 'filename=' parameter
+           (encoded filenames (*=) are not supported) */
+        for(;;) {
+          char *filename;
+          size_t len;
 
-      while((p < end) && *p && !ISALPHA(*p))
-        p++;
-      if(p > end - 9)
-        break;
+          while((p < end) && *p && !ISALPHA(*p))
+            p++;
+          if(p > end - 9)
+            break;
 
-      if(memcmp(p, "filename=", 9)) {
-        /* no match, find next parameter */
-        while((p < end) && *p && (*p != ';'))
-          p++;
-        if((p < end) && *p)
-          continue;
-        else
+          if(memcmp(p, "filename=", 9)) {
+            /* no match, find next parameter */
+            while((p < end) && *p && (*p != ';'))
+              p++;
+            if((p < end) && *p)
+              continue;
+            else
+              break;
+          }
+          p += 9;
+
+          len = cb - (size_t)(p - str);
+          filename = parse_filename(p, len);
+          if(filename) {
+            if(outs->stream) {
+              /* indication of problem, get out! */
+              free(filename);
+              return CURL_WRITEFUNC_ERROR;
+            }
+
+            if(per->config->output_dir) {
+              outs->filename = aprintf("%s/%s", per->config->output_dir,
+                                       filename);
+              free(filename);
+              if(!outs->filename)
+                return CURL_WRITEFUNC_ERROR;
+            }
+            else
+              outs->filename = filename;
+
+            outs->is_cd_filename = TRUE;
+            outs->s_isreg = TRUE;
+            outs->fopened = FALSE;
+            outs->alloc_filename = TRUE;
+            hdrcbdata->honor_cd_filename = FALSE; /* done now! */
+            if(!tool_create_output_file(outs, per->config))
+              return CURL_WRITEFUNC_ERROR;
+            if(tool_write_headers(&per->hdrcbdata, outs->stream))
+              return CURL_WRITEFUNC_ERROR;
+          }
           break;
-      }
-      p += 9;
+        }
+        if(!outs->stream && !tool_create_output_file(outs, per->config))
+          return CURL_WRITEFUNC_ERROR;
+        if(tool_write_headers(&per->hdrcbdata, outs->stream))
+          return CURL_WRITEFUNC_ERROR;
+      } /* content-disposition handling */
 
-      /* this expression below typecasts 'cb' only to avoid
-         warning: signed and unsigned type in conditional expression
-      */
-      len = (ssize_t)cb - (p - str);
-      filename = parse_filename(p, len);
-      if(filename) {
-        if(outs->stream) {
-          /* indication of problem, get out! */
-          free(filename);
+      if(hdrcbdata->honor_cd_filename &&
+         hdrcbdata->config->show_headers) {
+        /* still awaiting the Content-Disposition header, store the header in
+           memory. Since it is not zero terminated, we need an extra dance. */
+        char *clone = aprintf("%.*s", (int)cb, (char *)str);
+        if(clone) {
+          struct curl_slist *old = hdrcbdata->headlist;
+          hdrcbdata->headlist = curl_slist_append(old, clone);
+          free(clone);
+          if(!hdrcbdata->headlist) {
+            curl_slist_free_all(old);
+            return CURL_WRITEFUNC_ERROR;
+          }
+        }
+        else {
+          curl_slist_free_all(hdrcbdata->headlist);
+          hdrcbdata->headlist = NULL;
           return CURL_WRITEFUNC_ERROR;
         }
-
-        outs->is_cd_filename = TRUE;
-        outs->s_isreg = TRUE;
-        outs->fopened = FALSE;
-        outs->filename = filename;
-        outs->alloc_filename = TRUE;
-        hdrcbdata->honor_cd_filename = FALSE; /* done now! */
-        if(!tool_create_output_file(outs, per->config))
-          return CURL_WRITEFUNC_ERROR;
+        return cb; /* done for now */
       }
-      break;
     }
-    if(!outs->stream && !tool_create_output_file(outs, per->config))
-      return CURL_WRITEFUNC_ERROR;
   }
   if(hdrcbdata->config->writeout) {
     char *value = memchr(ptr, ':', cb);
@@ -240,7 +317,7 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
 }
 
 /*
- * Copies a file name part and returns an ALLOCATED data buffer.
+ * Copies a filename part and returns an ALLOCATED data buffer.
  */
 static char *parse_filename(const char *ptr, size_t len)
 {
@@ -281,7 +358,7 @@ static char *parse_filename(const char *ptr, size_t len)
   }
 
   /* If the filename contains a backslash, only use filename portion. The idea
-     is that even systems that don't handle backslashes as path separators
+     is that even systems that do not handle backslashes as path separators
      probably want the path removed for convenience. */
   q = strrchr(p, '\\');
   if(q) {
@@ -292,7 +369,7 @@ static char *parse_filename(const char *ptr, size_t len)
     }
   }
 
-  /* make sure the file name doesn't end in \r or \n */
+  /* make sure the filename does not end in \r or \n */
   q = strchr(p, '\r');
   if(q)
     *q = '\0';
@@ -315,26 +392,6 @@ static char *parse_filename(const char *ptr, size_t len)
   }
 #endif /* _WIN32 || MSDOS */
 
-  /* in case we built debug enabled, we allow an environment variable
-   * named CURL_TESTDIR to prefix the given file name to put it into a
-   * specific directory
-   */
-#ifdef DEBUGBUILD
-  {
-    char *tdir = curlx_getenv("CURL_TESTDIR");
-    if(tdir) {
-      char buffer[512]; /* suitably large */
-      msnprintf(buffer, sizeof(buffer), "%s/%s", tdir, copy);
-      Curl_safefree(copy);
-      copy = strdup(buffer); /* clone the buffer, we don't use the libcurl
-                                aprintf() or similar since we want to use the
-                                same memory code as the "real" parse_filename
-                                function */
-      curl_free(tdir);
-    }
-  }
-#endif
-
   return copy;
 }
 
@@ -343,9 +400,9 @@ static char *parse_filename(const char *ptr, size_t len)
  * Treat the Location: header specially, by writing a special escape
  * sequence that adds a hyperlink to the displayed text. This makes
  * the absolute URL of the redirect clickable in supported terminals,
- * which couldn't happen otherwise for relative URLs. The Location:
+ * which could not happen otherwise for relative URLs. The Location:
  * header is supposed to always be absolute so this theoretically
- * shouldn't be needed but the real world returns plenty of relative
+ * should not be needed but the real world returns plenty of relative
  * URLs here.
  */
 static
@@ -417,7 +474,7 @@ void write_linked_location(CURL *curl, const char *location, size_t loclen,
     goto locdone;
   }
 
-  /* Not a "safe" URL: don't linkify it */
+  /* Not a "safe" URL: do not linkify it */
 
 locout:
   /* Write the normal output in case of error or unsafe */
