@@ -21,12 +21,11 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#define CURL_DISABLE_DEPRECATION  /* Using and testing the form api */
 #include "test.h"
 
 #include "memdebug.h"
 
-static char data[]=
+static char testdata[]=
   "this is what we post to the silly web server\n";
 
 struct WriteThis {
@@ -60,7 +59,7 @@ static size_t read_callback(char *ptr, size_t size, size_t nmemb, void *userp)
 #endif
 }
 
-static int once(char *URL, bool oldstyle)
+static CURLcode test_once(char *URL, bool oldstyle)
 {
   CURL *curl;
   CURLcode res = CURLE_OK;
@@ -71,28 +70,32 @@ static int once(char *URL, bool oldstyle)
   struct WriteThis pooh;
   struct WriteThis pooh2;
 
-  pooh.readptr = data;
-  pooh.sizeleft = strlen(data);
+  pooh.readptr = testdata;
+  pooh.sizeleft = strlen(testdata);
 
   /* Fill in the file upload field */
   if(oldstyle) {
-    formrc = curl_formadd(&formpost,
-                          &lastptr,
-                          CURLFORM_COPYNAME, "sendfile",
-                          CURLFORM_STREAM, &pooh,
-                          CURLFORM_CONTENTSLENGTH, (long)pooh.sizeleft,
-                          CURLFORM_FILENAME, "postit2.c",
-                          CURLFORM_END);
+    CURL_IGNORE_DEPRECATION(
+      formrc = curl_formadd(&formpost,
+                            &lastptr,
+                            CURLFORM_COPYNAME, "sendfile",
+                            CURLFORM_STREAM, &pooh,
+                            CURLFORM_CONTENTSLENGTH, (long)pooh.sizeleft,
+                            CURLFORM_FILENAME, "postit2.c",
+                            CURLFORM_END);
+    )
   }
   else {
-    /* new style */
-    formrc = curl_formadd(&formpost,
-                          &lastptr,
-                          CURLFORM_COPYNAME, "sendfile alternative",
-                          CURLFORM_STREAM, &pooh,
-                          CURLFORM_CONTENTLEN, (curl_off_t)pooh.sizeleft,
-                          CURLFORM_FILENAME, "file name 2",
-                          CURLFORM_END);
+    CURL_IGNORE_DEPRECATION(
+      /* new style */
+      formrc = curl_formadd(&formpost,
+                            &lastptr,
+                            CURLFORM_COPYNAME, "sendfile alternative",
+                            CURLFORM_STREAM, &pooh,
+                            CURLFORM_CONTENTLEN, (curl_off_t)pooh.sizeleft,
+                            CURLFORM_FILENAME, "file name 2",
+                            CURLFORM_END);
+    )
   }
 
   if(formrc)
@@ -101,55 +104,61 @@ static int once(char *URL, bool oldstyle)
   /* Now add the same data with another name and make it not look like
      a file upload but still using the callback */
 
-  pooh2.readptr = data;
-  pooh2.sizeleft = strlen(data);
-
-  /* Fill in the file upload field */
-  formrc = curl_formadd(&formpost,
-                        &lastptr,
-                        CURLFORM_COPYNAME, "callbackdata",
-                        CURLFORM_STREAM, &pooh2,
-                        CURLFORM_CONTENTSLENGTH, (long)pooh2.sizeleft,
-                        CURLFORM_END);
+  pooh2.readptr = testdata;
+  pooh2.sizeleft = strlen(testdata);
 
+  CURL_IGNORE_DEPRECATION(
+    /* Fill in the file upload field */
+    formrc = curl_formadd(&formpost,
+                          &lastptr,
+                          CURLFORM_COPYNAME, "callbackdata",
+                          CURLFORM_STREAM, &pooh2,
+                          CURLFORM_CONTENTSLENGTH, (long)pooh2.sizeleft,
+                          CURLFORM_END);
+  )
   if(formrc)
     printf("curl_formadd(2) = %d\n", (int)formrc);
 
-  /* Fill in the filename field */
-  formrc = curl_formadd(&formpost,
-                        &lastptr,
-                        CURLFORM_COPYNAME, "filename",
-                        CURLFORM_COPYCONTENTS, "postit2.c",
-                        CURLFORM_END);
-
+  CURL_IGNORE_DEPRECATION(
+    /* Fill in the filename field */
+    formrc = curl_formadd(&formpost,
+                          &lastptr,
+                          CURLFORM_COPYNAME, "filename",
+                          CURLFORM_COPYCONTENTS, "postit2.c",
+                          CURLFORM_END);
+  )
   if(formrc)
     printf("curl_formadd(3) = %d\n", (int)formrc);
 
-  /* Fill in a submit field too */
-  formrc = curl_formadd(&formpost,
-                        &lastptr,
-                        CURLFORM_COPYNAME, "submit",
-                        CURLFORM_COPYCONTENTS, "send",
-                        CURLFORM_CONTENTTYPE, "text/plain",
-                        CURLFORM_END);
-
+  CURL_IGNORE_DEPRECATION(
+    /* Fill in a submit field too */
+    formrc = curl_formadd(&formpost,
+                          &lastptr,
+                          CURLFORM_COPYNAME, "submit",
+                          CURLFORM_COPYCONTENTS, "send",
+                          CURLFORM_CONTENTTYPE, "text/plain",
+                          CURLFORM_END);
+  )
   if(formrc)
     printf("curl_formadd(4) = %d\n", (int)formrc);
 
-  formrc = curl_formadd(&formpost, &lastptr,
-                        CURLFORM_COPYNAME, "somename",
-                        CURLFORM_BUFFER, "somefile.txt",
-                        CURLFORM_BUFFERPTR, "blah blah",
-                        CURLFORM_BUFFERLENGTH, (long)9,
-                        CURLFORM_END);
-
+  CURL_IGNORE_DEPRECATION(
+    formrc = curl_formadd(&formpost, &lastptr,
+                          CURLFORM_COPYNAME, "somename",
+                          CURLFORM_BUFFER, "somefile.txt",
+                          CURLFORM_BUFFERPTR, "blah blah",
+                          CURLFORM_BUFFERLENGTH, (long)9,
+                          CURLFORM_END);
+  )
   if(formrc)
     printf("curl_formadd(5) = %d\n", (int)formrc);
 
   curl = curl_easy_init();
   if(!curl) {
     fprintf(stderr, "curl_easy_init() failed\n");
-    curl_formfree(formpost);
+    CURL_IGNORE_DEPRECATION(
+      curl_formfree(formpost);
+    )
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
   }
@@ -166,8 +175,10 @@ static int once(char *URL, bool oldstyle)
   /* we want to use our own read function */
   test_setopt(curl, CURLOPT_READFUNCTION, read_callback);
 
-  /* send a multi-part formpost */
-  test_setopt(curl, CURLOPT_HTTPPOST, formpost);
+  CURL_IGNORE_DEPRECATION(
+    /* send a multi-part formpost */
+    test_setopt(curl, CURLOPT_HTTPPOST, formpost);
+  )
 
   /* get verbose debug output please */
   test_setopt(curl, CURLOPT_VERBOSE, 1L);
@@ -180,27 +191,31 @@ static int once(char *URL, bool oldstyle)
 
 test_cleanup:
 
-  /* always cleanup */
-  curl_easy_cleanup(curl);
+  CURL_IGNORE_DEPRECATION(
+    /* always cleanup */
+    curl_easy_cleanup(curl);
+  )
 
-  /* now cleanup the formpost chain */
-  curl_formfree(formpost);
+  CURL_IGNORE_DEPRECATION(
+    /* now cleanup the formpost chain */
+    curl_formfree(formpost);
+  )
 
   return res;
 }
 
-int test(char *URL)
+CURLcode test(char *URL)
 {
-  int res;
+  CURLcode res;
 
   if(curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
     fprintf(stderr, "curl_global_init() failed\n");
     return TEST_ERR_MAJOR_BAD;
   }
 
-  res = once(URL, TRUE); /* old */
+  res = test_once(URL, TRUE); /* old */
   if(!res)
-    res = once(URL, FALSE); /* new */
+    res = test_once(URL, FALSE); /* new */
 
   curl_global_cleanup();
 
