@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 
 /* This test case is supposed to be identical to 547 except that this uses the
@@ -36,14 +38,7 @@
 #define TEST_HANG_TIMEOUT 60 * 1000
 
 static const char uploadthis[] =
-#ifdef CURL_DOES_CONVERSIONS
-  /* ASCII representation with escape sequences for non-ASCII platforms */
-  "\x74\x68\x69\x73\x20\x69\x73\x20\x74\x68\x65\x20\x62\x6c\x75\x72"
-  "\x62\x20\x77\x65\x20\x77\x61\x6e\x74\x20\x74\x6f\x20\x75\x70\x6c"
-  "\x6f\x61\x64\x0a";
-#else
   "this is the blurb we want to upload\n";
-#endif
 
 static size_t readcallback(char  *ptr,
                            size_t size,
@@ -59,7 +54,7 @@ static size_t readcallback(char  *ptr,
   }
   (*counter)++; /* bump */
 
-  if(size * nmemb > strlen(uploadthis)) {
+  if(size * nmemb >= strlen(uploadthis)) {
     fprintf(stderr, "READ!\n");
     strcpy(ptr, uploadthis);
     return strlen(uploadthis);
@@ -81,9 +76,9 @@ static curlioerr ioctlcallback(CURL *handle,
 }
 
 
-int test(char *URL)
+CURLcode test(char *URL)
 {
-  int res = 0;
+  CURLcode res = CURLE_OK;
   CURL *curl = NULL;
   int counter = 0;
   CURLM *m = NULL;
@@ -100,8 +95,10 @@ int test(char *URL)
   easy_setopt(curl, CURLOPT_HEADER, 1L);
 
   /* read the POST data from a callback */
-  easy_setopt(curl, CURLOPT_IOCTLFUNCTION, ioctlcallback);
-  easy_setopt(curl, CURLOPT_IOCTLDATA, &counter);
+  CURL_IGNORE_DEPRECATION(
+    easy_setopt(curl, CURLOPT_IOCTLFUNCTION, ioctlcallback);
+    easy_setopt(curl, CURLOPT_IOCTLDATA, &counter);
+  )
   easy_setopt(curl, CURLOPT_READFUNCTION, readcallback);
   easy_setopt(curl, CURLOPT_READDATA, &counter);
   /* We CANNOT do the POST fine without setting the size (or choose
@@ -130,10 +127,6 @@ int test(char *URL)
 
     abort_on_test_timeout();
 
-#ifdef TPF
-    sleep(1); /* avoid ctl-10 dump */
-#endif
-
     if(!running)
       break; /* done */
 
