@@ -5,11 +5,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2016, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at https://curl.haxx.se/docs/copyright.html.
+ * are also available at https://curl.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "server_setup.h"
 
@@ -30,15 +32,15 @@
 
 int main(int argc, char **argv)
 {
-  int rc;
   char  *part;
-  size_t partlen, i;
+  size_t partlen;
 
-  if(argc< 3) {
+  if(argc < 3) {
     printf("./testpart main sub\n");
   }
   else {
-    rc = getpart(&part, &partlen, argv[1], argv[2], stdin);
+    int rc = getpart(&part, &partlen, argv[1], argv[2], stdin);
+    size_t i;
     if(rc)
       return rc;
     for(i = 0; i < partlen; i++)
@@ -47,4 +49,3 @@ int main(int argc, char **argv)
   }
   return 0;
 }
-
