@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "curlcheck.h"
 
@@ -39,20 +41,17 @@ static void unit_stop(void)
   (!defined(HAVE_FSETXATTR) && \
   (!defined(__FreeBSD_version) || (__FreeBSD_version < 500000)))
 UNITTEST_START
-{
-  return 0;
-}
 UNITTEST_STOP
 #else
 
-bool stripcredentials(char **url);
+char *stripcredentials(const char *url);
 
 struct checkthis {
   const char *input;
   const char *output;
 };
 
-static struct checkthis tests[] = {
+static const struct checkthis tests[] = {
   { "ninja://foo@example.com", "ninja://foo@example.com" },
   { "https://foo@example.com", "https://example.com/" },
   { "https://localhost:45", "https://localhost:45/" },
@@ -65,27 +64,18 @@ static struct checkthis tests[] = {
 
 UNITTEST_START
 {
-  bool cleanup;
-  char *url;
   int i;
-  int rc = 0;
 
   for(i = 0; tests[i].input; i++) {
-    url = (char *)tests[i].input;
-    cleanup = stripcredentials(&url);
+    const char *url = tests[i].input;
+    char *stripped = stripcredentials(url);
     printf("Test %u got input \"%s\", output: \"%s\"\n",
-           i, tests[i].input, url);
+           i, tests[i].input, stripped);
 
-    if(strcmp(tests[i].output, url)) {
-      fprintf(stderr, "Test %u got input \"%s\", expected output \"%s\"\n"
-              " Actual output: \"%s\"\n", i, tests[i].input, tests[i].output,
-              url);
-      rc++;
-    }
-    if(cleanup)
-      curl_free(url);
+    fail_if(stripped && strcmp(tests[i].output, stripped),
+            tests[i].output);
+    curl_free(stripped);
   }
-  return rc;
 }
 UNITTEST_STOP
 #endif
